# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: results_fields.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14results_fields.proto\x12\x1b\x61rmonik.api.grpc.v1.results\"P\n\x0eResultRawField\x12>\n\x05\x66ield\x18\x01 \x01(\x0e\x32/.armonik.api.grpc.v1.results.ResultRawEnumField\"_\n\x0bResultField\x12G\n\x10result_raw_field\x18\x01 \x01(\x0b\x32+.armonik.api.grpc.v1.results.ResultRawFieldH\x00\x42\x07\n\x05\x66ield*\xbf\x02\n\x12ResultRawEnumField\x12%\n!RESULT_RAW_ENUM_FIELD_UNSPECIFIED\x10\x00\x12$\n RESULT_RAW_ENUM_FIELD_SESSION_ID\x10\x01\x12\x1e\n\x1aRESULT_RAW_ENUM_FIELD_NAME\x10\x02\x12\'\n#RESULT_RAW_ENUM_FIELD_OWNER_TASK_ID\x10\x03\x12 \n\x1cRESULT_RAW_ENUM_FIELD_STATUS\x10\x04\x12$\n RESULT_RAW_ENUM_FIELD_CREATED_AT\x10\x05\x12&\n\"RESULT_RAW_ENUM_FIELD_COMPLETED_AT\x10\x06\x12#\n\x1fRESULT_RAW_ENUM_FIELD_RESULT_ID\x10\x07\x42\x1e\xaa\x02\x1b\x41rmoniK.Api.gRPC.V1.Resultsb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'results_fields_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\252\002\033ArmoniK.Api.gRPC.V1.Results'
  _globals['_RESULTRAWENUMFIELD']._serialized_start=233
  _globals['_RESULTRAWENUMFIELD']._serialized_end=552
  _globals['_RESULTRAWFIELD']._serialized_start=53
  _globals['_RESULTRAWFIELD']._serialized_end=133
  _globals['_RESULTFIELD']._serialized_start=135
  _globals['_RESULTFIELD']._serialized_end=230
# @@protoc_insertion_point(module_scope)
